package com.jty.wsxt.application.service.impl;

import com.jty.wsxt.application.service.PaperService;
import com.jty.wsxt.domain.DomainRegistry;
import com.jty.wsxt.domain.model.paper.Paper;
import com.jty.wsxt.infrastructure.support.BusinessException;
import com.jty.wsxt.infrastructure.support.ResultCode;
import com.jty.wsxt.interfaces.dto.PaperSearchDto;
import com.jty.wsxt.interfaces.dto.PaperSearchForCreatorDto;
import org.hibernate.query.criteria.internal.OrderImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSON;

import javax.persistence.criteria.*;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * PaperServiceImpl
 *
 * @author Manjiajie
 * @since 2019-6-21 11:38:06
 */
@Service
public class PaperServiceImpl implements PaperService {
    @Override
    public void addPaper(PaperSearchDto paperSearchDto) {
        Paper paper = new Paper();
        BeanUtils.copyProperties(paperSearchDto,paper,"answer","firstFeedback","secondFeedback");
        paper.setAnswer(JSON.toJSONString(paperSearchDto.getAnswer()));
        paper.save();
    }

    @Override
    public Page<Paper> findPagePaper(PaperSearchDto paperSearchDto) {
        Paper paper = new Paper();
        if(paperSearchDto.getModuleId() != null){
            paper.setModuleId(paperSearchDto.getModuleId());
        }
        if(paperSearchDto.getSubjectId() != null){
            paper.setSubjectId(paperSearchDto.getSubjectId());
        }
        if(paperSearchDto.getTypeId() != null){
            paper.setTypeId(paperSearchDto.getTypeId());
        }
        ExampleMatcher matcher = ExampleMatcher.matching().withMatcher("name",ExampleMatcher.GenericPropertyMatchers.contains());
        Example<Paper> example = Example.of(paper,matcher);

        return DomainRegistry.paperRepository().findAll(example,paperSearchDto.getPageable());
    }

    @Override
    public void deletePaper(Integer paperId) {
        Paper paper = this.findPaperDetailById(paperId);
        paper.delete();
    }

    @Override
    public Page<Paper> findPagePaperByCreatorId(PaperSearchForCreatorDto paperSearchDto) {
        return DomainRegistry.paperRepository().findAll(PaperDsl.getNrscTeachersWhereClause(paperSearchDto),paperSearchDto.getPageable());
    }

    @Override
    public Paper findPaperDetailById(Integer id) {
        return DomainRegistry.paperRepository().findById(id).orElseThrow(()->new BusinessException(ResultCode.RESULT_DATA_NONE));
    }

    @Override
    public List<String> getAnswerById(Integer id) {
        Paper paper = this.findPaperDetailById(id);
        return JSON.parseArray(paper.getAnswer(), String.class);
    }

    @Override
    public List<String> getFirstFeedbackById(Integer id) {
        Paper paper = this.findPaperDetailById(id);
        return JSON.parseArray(paper.getFirstFeedback(), String.class);
    }

    @Override
    public List<String> getSecondFeedbackById(Integer id) {
        Paper paper = this.findPaperDetailById(id);
        return JSON.parseArray(paper.getSecondFeedback(), String.class);
    }

    @Override
    public void firstFeedback(PaperSearchDto paperSearchDto) {
        Paper paper = this.findPaperDetailById(paperSearchDto.getId());
        paper.setFirstFeedback(JSON.toJSONString(paperSearchDto.getFirstFeedback()));
        paper.setFeedbackTime(LocalDateTime.now());
        paper.setSpecialistId(paperSearchDto.getSpecialistId());
        paper.setSpecialistName(paperSearchDto.getSpecialistName());
        paper.save();
    }

    @Override
    public void secondFeedback(PaperSearchDto paperSearchDto) {
        Paper paper = this.findPaperDetailById(paperSearchDto.getId());
        paper.setSecondFeedback(JSON.toJSONString(paperSearchDto.getSecondFeedback()));
        paper.setFeedbackTime(LocalDateTime.now());
        paper.setSpecialistId(paperSearchDto.getSpecialistId());
        paper.setSpecialistName(paperSearchDto.getSpecialistName());
        paper.save();
    }
}
class PaperDsl {
    public static Specification<Paper> getNrscTeachersWhereClause(PaperSearchForCreatorDto searchDto) {
        return new Specification<Paper>() {
            @Override
            public Predicate toPredicate(Root<Paper> root, CriteriaQuery<?> query,
                                         CriteriaBuilder cb) {
                List<Predicate> predicate = new ArrayList<>();
                predicate.add(cb.equal(root.get("creatorId"), searchDto.getCreatorId()));

                if(searchDto.getSubjectId() != null){
                    predicate.add(cb.equal(root.get("subjectId"), searchDto.getSubjectId()));
                }

                if(searchDto.getUploadingStartTime() != null) {
                    predicate.add(cb.greaterThanOrEqualTo(root.get("createTime"), searchDto.getUploadingStartTime()));
                }

                if(searchDto.getUploadingEndTime() != null) {
                    predicate.add(cb.lessThanOrEqualTo(root.get("createTime"), searchDto.getUploadingEndTime()));
                }

                List<Order> orders = new ArrayList<>();
                orders.add(new OrderImpl(root.get("feedbackTime")));
                orders.add(new OrderImpl(root.get("id")));
                Predicate[] pre = new Predicate[predicate.size()];
                return query.distinct(true).where(predicate.toArray(pre)).orderBy(orders).getRestriction();
            }
        };
    }
}