package com.jty.wsxt.interfaces.controller;

import com.jty.wsxt.application.ApplicationRegistry;
import com.jty.wsxt.application.feign.UserFeign;
import com.jty.wsxt.domain.model.paper.Paper;
import com.jty.wsxt.infrastructure.support.Result;
import com.jty.wsxt.interfaces.dto.PaperDto;
import com.jty.wsxt.interfaces.dto.PaperSearchDto;
import com.jty.wsxt.interfaces.dto.PaperSearchForCreatorDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * PaperController
 *
 * @author Manjiajie
 * @since 2019-6-21 13:42:12
 */
@RestController
public class PaperController {

    private final UserFeign userFeign;

    @Autowired
    public PaperController(UserFeign userFeign) {
        this.userFeign = userFeign;
    }

    @PostMapping("/paper")
    public Result addPaper(@RequestBody PaperSearchDto paperSearchDto){
        paperSearchDto.setCreatorId(userFeign.getLoginId());
        paperSearchDto.setCreatorName(userFeign.getLoginName());
        ApplicationRegistry.paperService().addPaper(paperSearchDto);
        return Result.success();
    }

    @GetMapping("/papers")
    public Result getPapers(PaperSearchDto paperSearchDto){
        Page<Paper> paperPage = ApplicationRegistry.paperService().findPagePaper(paperSearchDto);
        return this.getPaperDtoPage(paperPage);
    }

    @GetMapping("/papers/creator")
    public Result getPapersByCreatorId(PaperSearchForCreatorDto paperSearchDto){
        Page<Paper> paperPage = ApplicationRegistry.paperService().findPagePaperByCreatorId(paperSearchDto);
        return this.getPaperDtoPage(paperPage);
    }

    @GetMapping("/paper/{paperId}")
    public Result getPaperDetail(@PathVariable Integer paperId){
       Paper paper =  ApplicationRegistry.paperService().findPaperDetailById(paperId);
       return Result.success(paper.cashToPaperDto());
    }

    @GetMapping("/paper/download-first-feedback/{paperId}")
    public Result downloadFirstFeedback(@PathVariable Integer paperId){
        return Result.success(ApplicationRegistry.paperService().getFirstFeedbackById(paperId));
    }

    @GetMapping("/paper/download-second-feedback/{paperId}")
    public Result downloadSecondFeedback(@PathVariable Integer paperId){
        return Result.success(ApplicationRegistry.paperService().getSecondFeedbackById(paperId));
    }

    @GetMapping("/paper/download-answer/{paperId}")
    public Result downloadAnswer(@PathVariable Integer paperId){
        return Result.success(ApplicationRegistry.paperService().getAnswerById(paperId));
    }

    @DeleteMapping("/paper/{paperId}")
    public Result deletePaper(@PathVariable Integer paperId){
        ApplicationRegistry.paperService().deletePaper(paperId);
        return Result.success();
    }

    @PutMapping("/paper/firstFeedback")
    public Result firstFeedback(@RequestBody PaperSearchDto paperSearchDto){
        paperSearchDto.setSpecialistId(userFeign.getLoginId());
        paperSearchDto.setSpecialistName(userFeign.getLoginName());
        ApplicationRegistry.paperService().firstFeedback(paperSearchDto);
        return Result.success();
    }

    @PutMapping("/paper/secondFeedback")
    public Result secondFeedback(@RequestBody PaperSearchDto paperSearchDto){
        paperSearchDto.setSpecialistId(userFeign.getLoginId());
        paperSearchDto.setSpecialistName(userFeign.getLoginName());
        ApplicationRegistry.paperService().secondFeedback(paperSearchDto);
        return Result.success();
    }

    private Result getPaperDtoPage(Page<Paper> paperPage){
        List<PaperDto> paperDtoList = new ArrayList<>();
        if(paperPage.getTotalElements() != 0){
            paperPage.getContent().forEach(paper -> paperDtoList.add(paper.cashToPaperDto()));
        }
        return Result.success(new PageImpl<>(paperDtoList,paperPage.getPageable(),paperPage.getTotalElements()));
    }

}
